#ifndef STARNET_H 
#define STARNET_H 
void  EnableLocalHost(void);
void  AccessUsingXdmcp(void);
void  AugmentSelf(unsigned long addr);
void  AddLocalHosts(void);
void  ResetHosts(char  *display);
int  AddHost(struct  _Client *client,unsigned char  family,unsigned length,unsigned char  *pAddr);
int  RemoveHost(struct  _Client *client,unsigned char  family,unsigned length,unsigned char  *pAddr);
int  GetHosts(void **data,int  *pnHosts,int  *pLen,unsigned char  *pEnabled);
int  InvalidHost(unsigned char  *saddr,int  len);
int  ChangeAccessControl(struct  _Client *client,int  fEnabled);
unsigned long  inet_addr(char  *name);
void  CreateWellKnownSockets(void);
void  ResetWellKnownSockets(void);
void  reset_socket(void);
char  *ClientAuthorized(struct  _Client *client,unsigned short  proto_n,char  *auth_proto,unsigned short  string_n,char  *auth_string);
void  EstablishNewConnections(struct  _osComm * *poc);
int  CloseDownConnection(struct  _Client *client);
void  OnlyListenToOneClient(struct  _Client *client);
void  ListenToAllClients(void);
unsigned long  ClientNetAddr(struct  _Client *client);
void debug_print(void);
void  start_network(void);
int  ReadRequestFromClient(struct  _Client *client);
int  InsertFakeRequest(struct  _Client *client,char  *data,int  count);
void ResetCurrentRequest(struct  _Client *client);
int  FlushClient(struct  _osComm *oc,char  *extraBuf,int  extraCount);
void  FlushAllOutput(void);
void  FlushIfCriticalOutputPending(void);
void  SetCriticalOutputPending(void);
int  WriteToClient(struct  _Client *client,int  count,char  *buf);
void  CloseDownFileDescriptor(struct  _osComm *oc);
void  ResetOsBuffers(void);
int  WaitForSomething(int  *pClientsReady);
void  XdmcpUseMsg(void);
int  XdmcpOptions(int  argc,char  * *argv,int  i);
void  XdmcpInit(void);
void  XdmcpReset(void);
void  XdmcpOpenDisplay(int  sock,unsigned long  addr);
void  XdmcpCloseDisplay(int  sock);
void  XdmcpBlockHandler(long  *wt);
void  XdmcpWakeupHandler(void);
void XdmcpSelectHost(struct  sockaddr *host_sockaddr,struct  _ARRAY8 *AuthenticationName);
void XdmcpAddHost(struct  sockaddr *from,struct  _ARRAY8 *AuthenticationName,struct  _ARRAY8 *hostname,struct  _ARRAY8 *status);
void  XdmcpReceivePacket(int  fd);
void XdmcpDeadSession(char  *reason);
#endif 
